use mysql::prelude::*;
use mysql::*;
use mysql_common::frunk::{hlist_pat, HList};
use struct_field_names_as_array::FieldNamesAsArray;
use time::{PrimitiveDateTime, Time};

#[allow(non_snake_case)]
#[derive(Debug, PartialEq, FieldNamesAsArray)]
pub struct Running {
    id: u32,
    pub distance: f32,
    pub time: Time,
    pub datetime: PrimitiveDateTime,
}

type RowType = HList!(u32, f32, Time, PrimitiveDateTime);

impl Running {
    fn get_sql_fields() -> String {
        Running::FIELD_NAMES_AS_ARRAY
            .iter()
            .cloned()
            .intersperse(", ")
            .collect()
    }

    fn query_map_helper() -> (String, impl Fn(RowType) -> Running) {
        let sql_query = format!("SELECT {} from running", Self::get_sql_fields());
        println!("{}", sql_query);

        let construction_closure = |row: RowType| {
            let hlist_pat![id, distance, time, datetime] = row;
            Running {
                id,
                distance,
                time,
                datetime,
            }
        };

        (sql_query, construction_closure)
    }

    pub async fn get_running_rows(sql_pool: Pool) -> Result<Vec<Running>> {
        let mut conn = sql_pool
            .get_conn()
            .expect("Cannot establish database connection");

        let (running_query, running_closure) = Self::query_map_helper();
        let running_val: Vec<Running> = conn
            .query_map(running_query, running_closure)
            .expect("Data in database doesn't match the Running class");

        Ok(running_val)
    }
}
