use mysql::prelude::*;
use mysql::*;
use mysql_common::frunk::{hlist_pat, HList};
use struct_field_names_as_array::FieldNamesAsArray;
use time::PrimitiveDateTime;

#[allow(non_snake_case)]
#[derive(Debug, PartialEq, FieldNamesAsArray)]
pub struct Weight {
    id: u32,
    pub weight: f32,
    pub datetime: PrimitiveDateTime,
}

type RowType = HList!(u32, f32, PrimitiveDateTime);

impl Weight {
    fn get_sql_fields() -> String {
        Weight::FIELD_NAMES_AS_ARRAY
            .iter()
            .cloned()
            .intersperse(", ")
            .collect()
    }

    fn query_map_helper() -> (String, impl Fn(RowType) -> Weight) {
        let sql_query = format!("SELECT {} from weight", Self::get_sql_fields());
        println!("{}", sql_query);

        let construction_closure = |row: RowType| {
            let hlist_pat![id, weight, datetime] = row;
            Weight {
                id,
                weight,
                datetime,
            }
        };

        (sql_query, construction_closure)
    }

    pub async fn get_weight_rows(sql_pool: Pool) -> Result<Vec<Weight>> {
        let mut conn = sql_pool
            .get_conn()
            .expect("Cannot establish database connection");

        let (weight_query, weight_closure) = Self::query_map_helper();
        let weight_val: Vec<Weight> = conn
            .query_map(weight_query, weight_closure)
            .expect("Data in database doesn't match the Weight class");

        Ok(weight_val)
    }
}
