// use super::plotter::Plotter;
use axum::http::header;
use axum::response::IntoResponse;
use plotters::prelude::*;

pub struct SVGPlotter {}
pub type SVGPlotRoot<'a> =
    plotters::drawing::DrawingArea<plotters::backend::SVGBackend<'a>, plotters::coord::Shift>;
pub type SVGPlotFun = Box<dyn Fn(SVGPlotRoot)>;

impl SVGPlotter {
    pub fn examplefun() -> SVGPlotFun {
        Box::new(|root: SVGPlotRoot| {
            root.fill(&WHITE).unwrap();
            let mut chart = ChartBuilder::on(&root)
                .caption("y=x^2", ("sans-serif", 50).into_font())
                .margin(5)
                .x_label_area_size(30)
                .y_label_area_size(30)
                .build_cartesian_2d(-1f32..1f32, -0.1f32..1f32)
                .unwrap();

            chart.configure_mesh().draw().unwrap();

            chart
                .draw_series(LineSeries::new(
                    (-50..=50).map(|x| x as f32 / 50.0).map(|x| (x, x * x)),
                    &RED,
                ))
                .unwrap()
                .label("y = x^2")
                .legend(|(x, y)| PathElement::new(vec![(x, y), (x + 20, y)], RED));

            chart
                .configure_series_labels()
                .background_style(WHITE.mix(0.8))
                .border_style(BLACK)
                .draw()
                .unwrap();

            root.present().unwrap();
        })
    }

    pub fn plot(width: u32, height: u32, plotting_fun: SVGPlotFun) -> impl IntoResponse {
        let mut image_string = String::new();

        {
            let root =
                SVGBackend::with_string(&mut image_string, (width, height)).into_drawing_area();
            plotting_fun(root);
        }

        let headers = [
            (header::CONTENT_TYPE, "image/svg+xml"),
            (header::CONTENT_DISPOSITION, "inline"),
        ];
        (headers, image_string).into_response()
    }
}
